==> wyvern-2.2.7.tar.bz2: wyvern-2.2.7/modules/mod_gzip.c <==
/*****************************************************************************
 *
 * FILE:	mod_gzip.c
 * DESCRIPTION:	Wyvern: gzip module using the zlib compression library
 * DATE:	Thu, May 17 2001
 * UPDATED:	Wed, Mar  1 2006
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2001-2006 졿Kouichi ABE (WALL), All rights reserved.
 * NOTE:	Based on minigzip.c
 * LICENSE:
 *
 *  Copyright (c) 2001-2006 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mod_gzip.c,v 1.17 2006/03/01 13:21:37 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_FCNTL_H
#include <fcntl.h>
#endif	/* HAVE_FCNTL_H */
#if	HAVE_MMAP
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#endif	/* HAVE_MMAP */
#include "wyvern.h"
#include "zlib.h"	/* collision of free_func used with OpenSSL */

/*****************************************************************************
 *
 *	Macros definition
 *
 *****************************************************************************/
#define	WM_MODULE_ID	"mod_gzip"

#define	GZIP_MODE	"wb6"	/* See zlib.h */
#define GZIP_BUFSIZE	(BUF_SIZE*4)
#define	MIN_FILE_SIZE	(512)

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static int	wm_gzip_encoding(Message *);
static int	gzip_compress(Message *, gzFile);
#if	HAVE_MMAP
static int	gzip_compress_with_mmap(Message *, gzFile);
#endif	/* HAVE_MMAP */
static int	reopen(Message *, const int);
static void	wm_gzip_register_hooks(void);

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * Function Name:	wm_gzip_encoding
 * Description:		compress content file with gzip compress
 * Arguments:		message
 * Return Value:	successful: 0, otherwise: -1
 */
static int
wm_gzip_encoding(msg)
	register Message *	msg;
{
  register const char *	p;
  int			fd;
  gzFile		out;

#if	1
  /*
   * Does OpenSSL conflict with zlib package?
   * I found the conflict name as "free_func" in both zlib.h and ssl.h etc.
   */
  if (msg->soci.secure == SOCKET_SECURE) {
    return WM_RET_OK;	/* no gzipped */
  }
#endif
  if (msg->response.status > STATUS_CODE_3XX ||
      msg->response.status == STATUS_PARTIAL_CONTENT) {
    return WM_RET_OK;	/* no gzipped */
  }

  /* No supported by client */
  if ((p = wa_get(msg->request.header, "Accept-Encoding")) == NULL ||
      strstr(p, "gzip") == NULL) {
    return WM_RET_OK;	/* no gzipped */
  }
  /* content size < MIN_FILE_SIZE */
  if (msg->file.status.st_size < MIN_FILE_SIZE) {
    return WM_RET_OK;	/* no gzipped */
  }

  /* Open gzip'ed output file */
  fd = wu_mkstemp(httpd->tmpDir);
  if (fd == -1) {
    return WM_RET_ERROR;
  }

  out = gzdopen(dup(fd), GZIP_MODE);
  if (out == NULL) {
    close(fd);
    return WM_RET_ERROR;
  }
#if	HAVE_MMAP
  /*
   * Try first compressing with mmap.
   * If mmap fails, use the normal fread loop.
   */
  if (gzip_compress_with_mmap(msg, out) != Z_OK &&
      gzip_compress(msg, out) != Z_OK) {
    return WM_RET_ERROR;
  }
#else
  if (gzip_compress(msg, out) != Z_OK) {
    return WM_RET_ERROR;
  }
#endif	/* HAVE_MMAP */

  return reopen(msg, fd);
}

/*
 * Function Name:	gzip_compress
 * Description:		compress content file with gzip compress
 * Arguments:		message, gzip'ed file pointer
 * Return Value:	successful: Z_OK, otherwise: Z_ERRNO
 */
static int
gzip_compress(msg, out)
	register Message *	msg;
	gzFile			out;
{
  char		buf[GZIP_BUFSIZE];
  size_t	len;
  int		retval = Z_OK;

  memset(buf, 0, sizeof(buf));
  wt_netfd_seek(msg->file.nfd, (off_t)msg->file.offset, WT_SEEK_SET);
  while ((len = wt_netfd_read(msg->file.nfd, buf, sizeof(buf),
			      WT_UTIME_NO_TIMEOUT)) > 0) {
    if (gzwrite(out, buf, (unsigned)len) != len) {
      retval = Z_ERRNO;
      break;
    }
    memset(buf, 0, sizeof(buf));
  }

  /* Close the compressed content file */
  if (gzclose(out) != Z_OK) {
    retval = Z_ERRNO;
  }

  return retval;
}

#if	HAVE_MMAP
/*
 * Function Name:	gzip_compress_with_mmap
 * Description:		compress content file with gzip compress using mmap
 * Arguments:		message, gzip'ed file pointer
 * Return Value:	successful: Z_OK, otherwise: Z_ERRNO
 */
static int
gzip_compress_with_mmap(msg, out)
	register Message *	msg;
	gzFile			out;
{
  caddr_t	buf;	/* mmap'ed buffer for the entire input file */
  size_t	len;
  int		retval = Z_OK;

  if (msg->file.status.st_size <= 0) {
    return Z_ERRNO;
  }

  /* Now do the actual mmap: */
  buf = mmap((caddr_t) 0, (size_t)msg->file.status.st_size, PROT_READ,
	      MAP_PRIVATE, wt_netfd_fileno(msg->file.nfd), msg->file.offset); 
  if (buf == MAP_FAILED) {
    return Z_ERRNO;
  }

  /* Compress the whole file at once: */
  len = gzwrite(out, (char *)buf, (unsigned)msg->file.status.st_size);
  if (len != (int)msg->file.status.st_size) {
    retval = Z_ERRNO;
  }
  munmap(buf, (size_t)msg->file.status.st_size);

  if (gzclose(out) != Z_OK) {
    retval = Z_ERRNO;
  }

  return retval;
}
#endif /* HAVE_MMAP */

/*
 * Function Name:	reopen
 * Description:		reopen file of content
 * Arguments:		message, gzip'ed file
 * Return Value:	successful: 0, otherwise: -1
 */
static int
reopen(msg, fd)
	register Message *	msg;
	const int		fd;
{
  struct stat	sbuf;

  /* Get file status */
  if (fstat(fd, &sbuf) == -1) {
    /* rewind */
    wt_netfd_seek(msg->file.nfd, (off_t)msg->file.offset, WT_SEEK_SET);
    return WM_RET_ERROR;
  }

  /* Change file descriptor from old to new */
  wa_put(msg->response.header, "Content-Encoding", "gzip");

  /* The file will be removed when it's closed. */
  wt_netfd_close(msg->file.nfd);
  msg->file.nfd			= wt_netfd_fdopen(fd);
  msg->file.offset		= 0;
  msg->file.status.st_size	= sbuf.st_size;

  return WM_RET_OK;
}

/*****************************************************************************
 *
 *	Module
 *
 *****************************************************************************/
static void
wm_gzip_register_hooks(void)
{
  wm_hook_contents_filter(wm_gzip_encoding, HOOK_FIXUP_RESPONSE, "text/*");
}

static wmModule	mod_gzip = {
  WM_MAGIC_NUMBER_MAJOR,		/* version */
  WM_SECURE,				/* secure */
  WM_MODULE_ID,				/* name */
  "GZip Module version 1.1.3",		/* info */
  "Kouichi ABE (WALL)",			/* author */
  NULL,					/* depend */
  NULL,					/* handle */
  NULL,					/* init */
  NULL,					/* finish */
  (void *)wm_gzip_register_hooks	/* register_handler */
};

wmModule *
wmModuleInfo(void)
{
  return &mod_gzip;
}

==> wyvern-2.2.10.tar.bz2: wyvern-2.2.10/modules/mod_gzip.c <==
/*****************************************************************************
 *
 * FILE:	mod_gzip.c
 * DESCRIPTION:	Wyvern: gzip module using the zlib compression library
 * DATE:	Thu, May 17 2001
 * UPDATED:	Wed, Sep 12 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2001-2007 졿Kouichi ABE (WALL), All rights reserved.
 * NOTE:	Based on minigzip.c
 * LICENSE:
 *
 *  Copyright (c) 2001-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mod_gzip.c,v 1.20 2007/09/12 02:24:26 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_FCNTL_H
#include <fcntl.h>
#endif	/* HAVE_FCNTL_H */
#if	HAVE_MMAP
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#endif	/* HAVE_MMAP */
#include "wyvern.h"
#include "zlib.h"	/* collision of free_func used with OpenSSL */

/*****************************************************************************
 *
 *	Macros definition
 *
 *****************************************************************************/
#define	WM_MODULE_ID	"mod_gzip"

#define	GZIP_MODE	"wb6"	/* See zlib.h */
#define GZIP_BUFSIZE	(BUF_SIZE*4)
#define	MIN_FILE_SIZE	(512)

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static int	wm_gzip_encoding(Message *);
static int	gzip_compress(Message *, gzFile);
#if	HAVE_MMAP
static int	gzip_compress_with_mmap(Message *, gzFile);
#endif	/* HAVE_MMAP */
static int	reopen(Message *, const int);
static void	wm_gzip_register_hooks(void);

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * Function Name:	wm_gzip_encoding
 * Description:		compress content file with gzip compress
 * Arguments:		message
 * Return Value:	successful: 0, otherwise: -1
 */
static int
wm_gzip_encoding(msg)
	register Message *	msg;
{
  register const char *	p;
  int			fd;
  gzFile		out;

#if	1
  /*
   * Does OpenSSL conflict with zlib package?
   * I found the conflict name as "free_func" in both zlib.h and ssl.h etc.
   */
  if (msg->soci.secure == SOCKET_SECURE) {
    return WM_RET_OK;	/* no gzipped */
  }
#endif
  if (msg->response.status > STATUS_CODE_3XX ||
      msg->response.status == STATUS_PARTIAL_CONTENT) {
    return WM_RET_OK;	/* no gzipped */
  }

  /* No supported by client */
  if ((p = wa_get(msg->request.header, "accept-encoding")) == NULL ||
      strstr(p, "gzip") == NULL) {
    return WM_RET_OK;	/* no gzipped */
  }
  /* content size < MIN_FILE_SIZE */
  if (msg->file.status.st_size < MIN_FILE_SIZE) {
    return WM_RET_OK;	/* no gzipped */
  }

  /* Open gzip'ed output file */
  fd = wu_mkstemp(httpd->tmpDir);
  if (fd == -1) {
    return WM_RET_ERROR;
  }

  out = gzdopen(dup(fd), GZIP_MODE);
  if (out == NULL) {
    close(fd);
    return WM_RET_ERROR;
  }
#if	HAVE_MMAP
  /*
   * Try first compressing with mmap.
   * If mmap fails, use the normal fread loop.
   */
  if (gzip_compress_with_mmap(msg, out) != Z_OK &&
      gzip_compress(msg, out) != Z_OK) {
    return WM_RET_ERROR;
  }
#else
  if (gzip_compress(msg, out) != Z_OK) {
    return WM_RET_ERROR;
  }
#endif	/* HAVE_MMAP */

  return reopen(msg, fd);
}

/*
 * Function Name:	gzip_compress
 * Description:		compress content file with gzip compress
 * Arguments:		message, gzip'ed file pointer
 * Return Value:	successful: Z_OK, otherwise: Z_ERRNO
 */
static int
gzip_compress(msg, out)
	register Message *	msg;
	gzFile			out;
{
  char		buf[GZIP_BUFSIZE];
  ssize_t	len;
  int		status = Z_OK;

  memset(buf, 0, sizeof(buf));
  wt_netfd_seek(msg->file.nfd, (off_t)msg->file.offset, WT_SEEK_SET);
  while ((len = wt_netfd_read(msg->file.nfd, buf, sizeof(buf),
			      WT_UTIME_NO_TIMEOUT)) > 0) {
    if (gzwrite(out, buf, (unsigned)len) != len) {
      status = Z_ERRNO;
      break;
    }
    memset(buf, 0, sizeof(buf));
  }

  /* Close the compressed content file */
  if (gzclose(out) != Z_OK) {
    status = Z_ERRNO;
  }

  return status;
}

#if	HAVE_MMAP
/*
 * Function Name:	gzip_compress_with_mmap
 * Description:		compress content file with gzip compress using mmap
 * Arguments:		message, gzip'ed file pointer
 * Return Value:	successful: Z_OK, otherwise: Z_ERRNO
 */
static int
gzip_compress_with_mmap(msg, out)
	register Message *	msg;
	gzFile			out;
{
  caddr_t	buf;	/* mmap'ed buffer for the entire input file */
  int		len;
  int		status = Z_OK;

  if (msg->file.status.st_size <= 0) {
    return Z_ERRNO;
  }

  /* Now do the actual mmap: */
  buf = mmap((caddr_t) 0, (size_t)msg->file.status.st_size, PROT_READ,
	      MAP_PRIVATE, wt_netfd_fileno(msg->file.nfd), msg->file.offset); 
  if (buf == MAP_FAILED) {
    return Z_ERRNO;
  }

  /* Compress the whole file at once: */
  len = gzwrite(out, (char *)buf, (unsigned)msg->file.status.st_size);
  if (len != (int)msg->file.status.st_size) {
    status = Z_ERRNO;
  }
  munmap(buf, (size_t)msg->file.status.st_size);

  if (gzclose(out) != Z_OK) {
    status = Z_ERRNO;
  }

  return status;
}
#endif /* HAVE_MMAP */

/*
 * Function Name:	reopen
 * Description:		reopen file of content
 * Arguments:		message, gzip'ed file
 * Return Value:	successful: 0, otherwise: -1
 */
static int
reopen(msg, fd)
	register Message *	msg;
	const int		fd;
{
  struct stat	sbuf;

  /* Get file status */
  if (fstat(fd, &sbuf) == -1) {
    /* rewind */
    wt_netfd_seek(msg->file.nfd, (off_t)msg->file.offset, WT_SEEK_SET);
    return WM_RET_ERROR;
  }

  /* Change file descriptor from old to new */
  wa_put(msg->response.header, "Content-Encoding", "gzip");

  /* The file will be removed when it's closed. */
  wt_netfd_close(msg->file.nfd);
  msg->file.nfd			= wt_netfd_fdopen(fd);
  msg->file.offset		= 0;
  msg->file.status.st_size	= sbuf.st_size;

  return WM_RET_OK;
}

/*****************************************************************************
 *
 *	Module
 *
 *****************************************************************************/
static void
wm_gzip_register_hooks(void)
{
  wm_hook_contents_filter(wm_gzip_encoding, HOOK_FIXUP_RESPONSE, "text/*");
}

static wmModule	mod_gzip = {
  WM_MAGIC_NUMBER_MAJOR,		/* version */
  WM_SECURE,				/* secure */
  WM_MODULE_ID,				/* name */
  "GZip Module version 1.1.3",		/* info */
  "Kouichi ABE (WALL)",			/* author */
  NULL,					/* depend */
  NULL,					/* handle */
  NULL,					/* init */
  NULL,					/* finish */
  (void *)wm_gzip_register_hooks	/* register_handler */
};

wmModule *
wmModuleInfo(void)
{
  return &mod_gzip;
}
